/**
 * Copyright (c) 2006 TU Braunschweig. All rights reserved.
 *
 * @file
 * @author  (last commit) $Author: ohl $
 * @date    $Date: 2006-10-19 17:30:05 +0000 (Thu, 19 Oct 2006) $
 * @version $Revision: 301 $
 *
 */

#include <sstream>
#include <string>
#include <algorithm>
#include <stdexcept>
#include "Config.h"

using namespace std;

    Config::Config()
    {}

    Config::~Config()
    {
        SectionKeyValueMap::iterator it = m_data.begin();

        for(; it != m_data.end(); it++)
        {
            KeyValueMap *sec = it->second;
            delete sec;
        }
    }

    ostream& Config::save(ostream &s) const
    {
        SectionKeyValueMap::const_iterator it = m_data.begin();

        for(; it != m_data.end(); it++)
        {
            KeyValueMap *sec = it->second;
            s << "[" << it->first << "]" << endl;

            KeyValueMap::iterator it2 = sec->begin();

            for(;  it2 != sec->end(); it2++)
            {
                s << it2->first << "=" << it2->second << endl;
            }
        }

        return s;
    }

    istream& Config::load(istream &s)
    {
        string sectionName = "errornosectionwasassigned";

        while(!s.eof())
        {
            string line;
            getline(s, line);

            size_t index1 = line.find_first_of("[") + 1;
            size_t index2 = line.find_first_of("]");
            size_t sectionLength = index2 - index1;

            size_t delimiter = line.find_first_of("=");

            size_t commentStart = line.find_first_of("#");

            size_t keyLength = delimiter;
            size_t valueLength = line.length();

            if (commentStart != string::npos)
            {
                valueLength = commentStart - (delimiter + 1);
            }

            if(index1 != string::npos && index2 != string::npos && sectionLength > 0)
            {
                sectionName = line.substr(index1, sectionLength);
                // Case Insensitive Suchen ermöglichen.
                transform(sectionName.begin(), sectionName.end(), sectionName.begin(), ptr_fun(::tolower)); 
            }

            if (delimiter != string::npos && keyLength > 0 && valueLength > 0)
            {
                KeyValueMap *section;

                SectionKeyValueMap::iterator secIt = m_data.find(sectionName);

                if (secIt == m_data.end())
                {
                    section = new KeyValueMap();
                    m_data[sectionName] = section;
                }

                else
                {
                    section = m_data[sectionName];
                }

                string key = line.substr(0, keyLength);
                string value = line.substr(delimiter + 1, valueLength);

                // Case Insensitive Suchen ermöglichen.
                transform(key.begin(), key.end(), key.begin(), ptr_fun(::tolower)); 

                (*section)[key] = value;
            }
        }

        return s;
    }

    int Config::getIntValue(string section, string key) const
    {
        istringstream is(getStringValue(section, key));
        int val;
        is >> val;
        return val;
    }

    float Config::getFloatValue(string section, string key) const
    {
        istringstream is(getStringValue(section, key));
        float val;
        is >> val;
        return val;
    }

    double Config::getDoubleValue(string section, string key) const
    {
        istringstream is(getStringValue(section, key));
        double val;
        is >> val;
        return val;
    }

    string Config::getStringValue(string section, string key) const
    {
        // Case Insensitive Suchen ermöglichen.
        transform(section.begin(), section.end(), section.begin(), ptr_fun(::tolower)); 
        transform(key.begin(), key.end(), key.begin(), ptr_fun(::tolower)); 

        SectionKeyValueMap::const_iterator secIt = m_data.find(section);

        if(secIt != m_data.end())
        {
            KeyValueMap *sec = secIt->second;

            if (sec->find(key) != sec->end())
            {
                return (*sec)[key];
            }
        }

        throw runtime_error("Config: section oder key nicht gefunden");
    }

    void Config::setValue(string section, string key, string value)
    {
        // Case Insensitive Suchen ermöglichen.
        transform(section.begin(), section.end(), section.begin(), ptr_fun(::tolower)); 
        transform(key.begin(), key.end(), key.begin(), ptr_fun(::tolower)); 

        if (m_data.count(section) == 0)
        {
            m_data[section] = new KeyValueMap();
        }

        KeyValueMap *sec = m_data[section];
        (*sec)[key] = value;
    }

    void Config::removeValue(string section, string key)
    {
        // Case Insensitive Suchen ermöglichen.
        transform(section.begin(), section.end(), section.begin(), ptr_fun(::tolower)); 
        transform(key.begin(), key.end(), key.begin(), ptr_fun(::tolower)); 

        SectionKeyValueMap::iterator it = m_data.find(section);

        if (it != m_data.end())
        {
            KeyValueMap *sec = it->second;
            sec->erase(key);

            if (sec->size() == 0)
            {
                m_data.erase(section);
                delete sec;
            }
        }
    }

    vector<string> Config::getSectionNames()
    {
        vector<string> result;
        for (SectionKeyValueMap::const_iterator it = m_data.begin(); it != m_data.end(); it++)
        {
            result.push_back(it->first);
        }
        return result;
    }

    vector<string> Config::getKeyNames(string section)
    {
        vector<string> result;

        // Case Insensitive Suchen ermöglichen.
        transform(section.begin(), section.end(), section.begin(), ptr_fun(::tolower)); 

        SectionKeyValueMap::iterator it = m_data.find(section);

        if (it != m_data.end())
        {
            KeyValueMap *sec = it->second;
            for (KeyValueMap::const_iterator it2 = sec->begin(); it2 != sec->end(); it2++)
            {
                result.push_back(it2->first);
            }
        }
        return result;
    }
