#include <unistd.h>
#include <fcntl.h>
#include <sys/signal.h>
#include <sys/types.h>

#include "Serial.h"
	              
#define BAUDRATE B9600
#define MODEMDEVICE "/dev/ttyS0"

using namespace std;

Serial::Serial()
{
    m_ok = true;
    
    m_device = open( MODEMDEVICE, O_RDWR | O_NOCTTY );
    if (m_device < 0) m_ok = false;

    struct termios options;

    tcgetattr(m_device,&m_oldtio); /* save current port settings */
    bzero(&options, sizeof(options)); /* clear struct for new port settings */
    
    options.c_cflag = BAUDRATE | CLOCAL | CREAD;

    /* No flow control */
    options.c_cflag &= ~CRTSCTS;
    
    /* Serial setup 8N1 */
    options.c_cflag &= ~PARENB;
    options.c_cflag &= ~CSTOPB;
    options.c_cflag &= ~CSIZE;
    options.c_cflag |= CS8;

    /* Serial local mode (no cannoical io, no echo, no signals) */
    options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
    
    /* Serial input mode  */
    options.c_iflag = IGNPAR;
    options.c_iflag &= ~(IXON | IXOFF | IXANY);
    options.c_oflag = 0;

    //mit TIME = 0, MIN = 0 blocked das 'read' nicht, sondern liefert alle Zeichen im Eingangspuffer zurück, ggf. halt keine 
    options.c_cc[VTIME]    = 0;  
    options.c_cc[VMIN]     = 0;
    
    int res = tcflush(m_device, TCIFLUSH);  //Eingangs-Buffer leeren
    if (res < 0)
    {
        m_ok = false;
    }

    res = tcsetattr(m_device, TCSANOW, &options);  //Optionen setzen
    if (res < 0)
    {
        m_ok = false; 
    }
    
}

Serial::~Serial()
{
    tcsetattr(m_device,TCSANOW,&m_oldtio);  //alte Einstellungen zurückschreiben
    close(m_device);
}

string Serial::readData( void )
{
    char buf[255];
    int res = read(m_device,buf,255); 
    buf[res]=0;
    return (string)buf;
}

void Serial::writeData( string s )
{
    //cout << "sending:" << s << "." << endl;
    int ret = write( m_device, s.c_str(), s.length() );
    if ( ret != s.length() ) m_ok = false;
}

bool Serial::serialOK( )
{
    return m_ok;
}

void Serial::clear()
{
    string s;
    do {
        s = readData();
    } while (s.length() != 0);
}
