#include <iostream>
#include "SimulatedCamera.h"
#include "Config.h"
#include <fstream>
#include <sstream>

using namespace std;

    SimulatedCamera::SimulatedCamera( )
    {
        //config einlesen:
        Config cfg;
        ifstream ifs("config.txt");
        cfg.load(ifs);
        
        m_x = cfg.getDoubleValue("Fahrzeug","startX");
        m_y = cfg.getDoubleValue("Fahrzeug","startY");
        m_angle = cfg.getDoubleValue("Fahrzeug","startAngle");
        
        m_ballX = cfg.getDoubleValue("Ball","ballX");
        m_ballY = cfg.getDoubleValue("Ball","ballY");
        m_ballSize = cfg.getDoubleValue("Ball","ballSize");
        
        m_numberWalls = cfg.getIntValue("Waende","anzahl");
        m_walls = new Wall[m_numberWalls];
        
        for (int i=0; i<m_numberWalls; i++)
        {
            stringstream name;

            name << "wand" << i << "X1";
            m_walls[i].setX1( cfg.getDoubleValue("Waende",name.str()) );

            name.str("");
            name.clear();
            name << "wand" << i << "Y1";
            m_walls[i].setY1( cfg.getDoubleValue("Waende",name.str()) );

            name.str("");
            name.clear();
            name << "wand" << i << "X2";
            m_walls[i].setX2( cfg.getDoubleValue("Waende",name.str()) );

            name.str("");
            name.clear();
            name << "wand" << i << "Y2";
            m_walls[i].setY2( cfg.getDoubleValue("Waende",name.str()) );
        }
        
        m_depth = new float[640];
        
    }

    bool SimulatedCamera::cameraOK( void ) const
    {
        //sim. Cam. is always ok
        return true;
    }
    
    IplImage* SimulatedCamera::getImage( void )
    {
        float wallheight = 0.5;
        float magicNumber = 554.0;
        

        //Tiefenpuffer initialisieren
        for (int i=0; i<640; i++)
        {
            m_depth[i] = 1000.0f;
        }
        
        //Abstand Ball
        double dX = m_ballX-m_x;
        double dY = m_ballY-m_y;

        double ballDistance = sqrt( (dX*dX) + (dY*dY) );

        //Winkel zum Ball
        double dAngle = normalize(atan2( -dY, dX ) - m_angle);
        
        
        //Tiefenpuffer aufbauen
        for ( int i=0; i<m_numberWalls; i++)
        {
            dX = m_walls[i].getX1()-m_x;
            dY = m_walls[i].getY1()-m_y;
            double dAngleLeft = normalize( atan2( -dY, dX ) - m_angle);
            float leftDepth = sqrt( (dX*dX) + (dY*dY) );
                
            dX = m_walls[i].getX2()-m_x;
            dY = m_walls[i].getY2()-m_y;
            double dAngleRight = normalize( atan2( -dY, dX ) - m_angle);
            float rightDepth = sqrt( (dX*dX) + (dY*dY) );
            
            if (( fabs( dAngleLeft ) < M_PI / 3.0 ) && ( fabs( dAngleRight ) < M_PI / 3.0 ))
            {
                int xBildLeft =  320-(int)(tan(dAngleLeft)  * magicNumber );
                int xBildRight = 320-(int)(tan(dAngleRight) * magicNumber );
                if (xBildLeft > xBildRight)
                {
                    int temp = xBildLeft;
                    xBildLeft = xBildRight;
                    xBildRight = temp;
                    float tempf = leftDepth;
                    leftDepth = rightDepth;
                    rightDepth = tempf;
                }
                
                for (int j=0; j<xBildRight-xBildLeft; j++)
                {
                    float depth = leftDepth + (j * (rightDepth - leftDepth) / (xBildRight - xBildLeft));
                    if ( m_depth[j+xBildLeft] > depth )
                    {
                        m_depth[j+xBildLeft] = depth;
                    }
                    
                }

            }
            
        }
        
        
        //jetzt wird gemalt
        IplImage *img = cvCreateImage( getResolution(),8,3 );
        
        //Himmel und Boden
        cvRectangle( img, cvPoint(0,0), cvPoint( getResolution().width-1, 240), CV_RGB( 230,230,230 ), -1 );
        cvRectangle( img, cvPoint(0,240), cvPoint( getResolution().width-1, getResolution().height-1), CV_RGB( 215,215,150 ), -1 );
        
        //Wände die weiter weg sind als der Ball
        for (int i=0; i<640; i++)
        {
            if ((m_depth[i] < 999.0 ) && ( m_depth[i] > ballDistance ))
            {
                int sizeDown = (int) (m_ballSize / m_depth[i] * magicNumber);
                int sizeUp   = (int) (wallheight / m_depth[i] * magicNumber);
                cvLine( img, cvPoint( i, 240-sizeUp), cvPoint(i,240+sizeDown), CV_RGB(111,87,1) ,1);
            }
        }
        
        //den Ball
        if ( fabs( dAngle ) < M_PI / 4.0 )
        {
            int xBallBild = 320-(int)(tan(dAngle) * 554.0);
            int sizeBallBild = int(m_ballSize / ballDistance * magicNumber);
            cvCircle( img, cvPoint(xBallBild,240 ), sizeBallBild, CV_RGB( 255,50,50), -1 );
        }

        //Wände die dichter dran sind als der Ball
        for (int i=0; i<640; i++)
        {
            if (m_depth[i] <  ballDistance )
            {
                int sizeDown = (int) (m_ballSize / m_depth[i] * magicNumber);
                int sizeUp   = (int) (wallheight / m_depth[i] * magicNumber);
                cvLine( img, cvPoint( i, 240-sizeUp), cvPoint(i,240+sizeDown), CV_RGB(111,87,1) ,1);
            }
        }
        
        //TODO Bildrauschen
        //CvRNG rng = cvRNG( 0x12345 );
        //cvRandArr( &rng, img, CV_RAND_NORMAL, cvScalar(0,0,0), cvScalar(256,256,256));
        
        
        return img;
    }
    
    void SimulatedCamera::move( float distance )
    {
        m_x += cos( m_angle ) * distance;
        m_y -= sin( m_angle ) * distance;
        
        //cout << m_x << "," << m_y << " (" << m_angle << ")"<< endl;            
    }
    
    void SimulatedCamera::turn( float angle )
    {
        m_angle += angle;
        //cout << m_x << "," << m_y << " (" << m_angle << ")"<< endl;            
    }        
    
    double SimulatedCamera::normalize( double a ) const
    {
        while ( a < -M_PI ) a+=2.0*M_PI;
        while ( a > M_PI  ) a-=2.0*M_PI;
        return a;
        
    }
